function AppController() {

    var self        = this;

    this.app        = {};
    this.user       = {};
    this.map        = {};
    this.service    = {};
    this.markers    = {};

    this.section    = null;
    this.redirect   = null;
    this.marker     = null;
    this.chat       = null;
    this.cache      = null;
    this.error      = null;

    this.loading    = false;
    this.results    = true;
    this.scroll     = true;

    this.page       = 1;
    this.channel    = 1;
    this.lastchat   = 0;

    this.initializeApp = function() {
        $.ajax({
            url: "https://m.weddyday.com/api/app.api.php",
            type: "GET",
            data: {
                action: "getApp"
            },
            dataType: "json",
            xhr: function() {
                return self.loadingProgress();
            }
        }).success( function( response ) {
            self.app = response;
        }).complete( function() {
            if (self.getToken() != null) {
                self.getUser();
            } else {
                self.loadSection( "main" );
            }
        });
    }

    this.getStock = function() {
        if (typeof self.app.stock == "object") {
            return self.app.stock;
        } else {
            return {};
        }
    }

    this.getKeywords = function() {
        if (typeof self.app.config.place[self.section] == "string") {
            return self.app.config.place[self.section];
        } else {
            return null;
        }
    }

    this.getPromotion = function( name ) {
        if (typeof self.app.config[self.section] == "object") {
            var promotion = null;
            $.each( self.app.config[self.section], function( promo, link ) {
                if (name.toLowerCase().indexOf( promo ) > -1) {
                    promotion = link;
                }
            });
            return promotion;
        } else {
            return null;
        }
    }

    this.scrollApp = function() {
        var scroll = $( window ).scrollTop();
        var client = window.innerHeight;
        var height = $( document ).height();
        if ($( "ul#products-" + self.section ).length > 0) {
            if ((scroll + client + 250) >= height) {
                self.loadProducts( self.page );
            }
        } else if ($( "div#chat" ).length > 0) {
            if ((scroll + client) < height) {
                self.unsetScroll();
            } else {
                self.setScroll();
            }
        }
    }

    this.openLink = function() {
        var link = $( this ).attr( "data-link" );
        window.open( link, "_blank" );
    }

    this.openAction = function() {
        window.location = $( this ).attr( "data-action" );
    }

    this.openMenu = function() {
        $( "div#menu" ).panel( "open" );
    }

    this.setSection = function( section ) {
        var section = $( this ).attr( "data-section" );
        if (section != self.section) {
            history.pushState({ section: section }, section );
        }
        self.loadSection( section );
        $( "div#menu" ).panel( "close" );
    }

    this.loadSection = function( section ) {
        self.section = section;
        self.authControl();
        $.ajax({
            url: "header/" + self.section + ".html",
            type: "GET",
            dataType: "html",
            beforeSend: function() {
                $( "a[data-section]" ).removeClass( "ui-btn-active" );
            },
            xhr: function() {
                return self.loadingProgress();
            }
        }).success( function( header ) {
            $.ajax({
                url: "pages/" + self.section + ".html",
                type: "GET",
                dataType: "html",
                beforeSend: function() {
                    $( "div[data-role='header']" ).html( header ).enhanceWithin();
                    $( "div[data-role='content']" ).empty();
                },
                xhr: function() {
                    return self.loadingProgress();
                }
            }).success( function( content ) {
                $( "div[data-role='content']" ).html( content ).enhanceWithin();
                switch( self.section ) {
                    case "main": {
                        if (self.getDate() != null) {
                            self.loadCountdown();
                        }
                        break;
                    }
                    case "chat": {
                        self.initializeChat();
                        break;
                    }
                    case "outfit": {
                        self.loadOutfit();
                        break;
                    }
                    case "accessories":
                    case "bridedress":
                    case "brideshoes":
                    case "citytrip":
                    case "cruise":
                    case "decoration":
                    case "invitations":
                    case "rings":
                    case "suit":
                    case "suitshoes":
                    case "thanksgivings":
                    case "travel": {
                        self.initializeProducts();
                        break;
                    }
                    case "bridebouquet":
                    case "cake":
                    case "car":
                    case "church":
                    case "entertainment":
                    case "hairstylist":
                    case "location":
                    case "photograph":
                    case "store":
                    case "theologian": {
                        if (self.getLocation() == null) {
                            self.toggleMapFilter();
                        }
                        self.loadMap();
                        break;
                    }
                    case "profile": {
                        self.loadProfile();
                        break;
                    }
                    case "todo": {
                        self.loadTodo();
                        $( "a[data-section='todo']" ).addClass( "ui-btn-active" );
                        break;
                    }
                    case "budget": {
                        self.loadBudget();
                        $( "a[data-section='budget']" ).addClass( "ui-btn-active" );
                        break;
                    }
                    case "guests": {
                        self.loadGuests();
                        $( "a[data-section='guests']" ).addClass( "ui-btn-active" );
                        break;
                    }
                }
            }).fail( function( error ) {
                self.fatalError( error.status );
            });
        }).fail( function( error ) {
            self.fatalError( error.status );
        });
    };

    this.loadLastSection = function() {
        if (typeof history.state.section == "string") {
            self.loadSection( history.state.section );
        } else {
            self.loadSection( "main" );
        }
        $( "div#menu" ).panel( "close" );
    }

    this.authControl = function() {
        switch( self.section ) {
            case "chat":
            case "profile":
            case "todo":
            case "budget":
            case "guests": {
                if (self.appUser() == false) {
                    self.redirect = self.section;
                    self.section = "login";
                }
                break;
            }
            case "logout": {
                self.logoutUser();
                self.section = "login";
                break;
            }
        }
    }

    this.loadCountdown = function() {
        $( "div#countdown" ).countdown( self.getDate(), { elapse: true } ).on( "update.countdown", function( event ) {
            $( "div#days" ).html( event.strftime( "<div>%-D</div>Tag%!D:e;" ) );
            $( "div#hours" ).html( event.strftime( "<div>%-H</div>Stunde%!H:n;" ) );
            $( "div#minutes" ).html( event.strftime( "<div>%-M</div>Minute%!M:n;" ) );
            $( "div#seconds" ).html( event.strftime( "<div>%-S</div>Sekunde%!S:n;" ) );
        }).fadeIn( 250 );
    }

    this.loadProfile = function() {
        if (self.getDate() != null) {
            var date = self.getDate().split( "-" );
            $( "input[name='date[day]']" ).val( date[2] );
            $( "input[name='date[month]']" ).val( date[1] );
            $( "input[name='date[year]']" ).val( date[0] );
        }
        if (self.getLocation() != null) {
            $( "span#profile-location" ).text( self.getLocation() );
        }
        $( "span#profile-guests" ).text( self.formatNumber( self.countGuests() ) );
        $( "span#profile-todo" ).text( self.formatNumber( self.countTodos() ) + " von 50" );
        $( "span#profile-budget" ).html( self.formatNumber( self.countBudgets() ) + " &euro;" );
        $( "span#profile-marker" ).html( self.formatNumber( self.countMarkers() ) );
        if (self.countMarkers() > 0) {
            $.each( self.getMarkers(), function( id, product ) {
                $( "ul#marker-products" ).append( product ).listview( "refresh" );
            });
            $( "a[data-marker]" ).addClass( "ui-btn-active" );
            self.monitorImages();
        } else {
            $( "ul#marker-products" ).html( "<li>Ihre Merkliste ist leer</li>" ).listview( "refresh" );
        }
    }

    this.focusProfile = function() {
        var profile = $( this ).attr( "data-profile" );
        $( "input[name^='" + profile + "']:first" ).focus();
    }

    this.focusDate = function() {
        $( this ).select();
    }

    this.loadTodo = function() {
        $.each( self.getTodos(), function() {
            $( "input[data-todo='" + this + "']" ).attr( "checked", true ).checkboxradio( "refresh" );
        });
    }

    this.loadBudget = function() {
        $.each( self.getBudgets(), function( budget, amount ) {
            $( "input[name='" + budget + "']" ).val( self.formatNumber( amount ) );
        });
        self.calculateBudget();
    }

    this.calculateBudget = function() {
        var totals = 0;
        $( "form#budgetlist" ).find( "ul" ).each( function() {
            var total = 0;
            $( this ).find( "input" ).each( function() {
                total += Number( this.value.replace( ".", "" ).replace( ",", "" ) );
            });
            $( this ).find( "span[data-budget='total']" ).html( self.formatNumber( total ) + " &euro;" );
            totals += total;
        });
        $( "span[data-budget='totals']" ).html( self.formatNumber( totals ) + " &euro;" );
    }

    this.focusBudget = function() {
        if ($( this ).attr( "data-budget" ) != undefined) {
            var budget = $( this ).attr( "data-budget" );
        } else {
            var budget = $( this ).attr( "name" );
        }
        $( "a[data-budget='" + budget + "']" ).addClass( "ui-btn-active" );
        $( "input[name='" + budget + "']" ).select();
    }

    this.loadGuests = function() {
        var scroll = 0, client = 0, height = 0;
        $.each( self.getGuests(), function( guest, name ) {
            var html = "<li><a data-name='" + name + "'><h2><span class='sort'></span>";
            if (name.match( /^\[(.*?)\]$/ ) != null) {
                html += name.match( /^\[(.*?)\]$/ )[1];
            } else {
                html += name;
            }
            html += "</h2></a><a data-guest='" + guest + "' class='ui-icon-delete'></a></li>";
            $( "ul#guests" ).append( html ).listview( "refresh" );
            if (name.match( /^\[(.*?)\]$/ ) != null) {
                $( "a[data-guest='" + guest + "']" ).parent().addClass( "ui-group-theme-b" );
            }
        });
        $( "ul#guests" ).sortable({
            containment: "parent",
            handle: "span.sort",
            placeholder: "sort",
            forcePlaceholderSize: true,
            opacity: 0.9,
            revert: true,
            start: function() {
                scroll = $( window ).scrollTop();
                client = window.innerHeight;
                height = $( document ).height();
            },
            create: function() {
                $( this ).height( $( this ).height() );
            },
            sort: function( event ) {
                if (event.clientY <= (client / 4)) {
                    if ((scroll - 5) > 0) {
                        scroll = scroll - 5;
                    } else {
                        scroll = 0;
                    }
                    $.mobile.silentScroll( scroll );
                } else if (event.clientY >= ((client / 4) * 3)) {
                    if ((scroll + 5) < (height - client)) {
                        scroll = scroll + 5;
                    } else {
                        scroll = height - client;
                    }
                    $.mobile.silentScroll( scroll );
                }
            }
        });
    }

    this.initializeChat = function() {
        self.setScroll();
        self.loading = false;
        self.loadChat( 0 );
        clearInterval( self.chat );
        self.chat = setInterval( function() {
            if ($( "div#chat" ).length > 0) {
                self.loadChat( self.lastchat );
            } else {
                clearInterval( self.chat );
            }
        }, 1000 );
    }

    this.loadChat = function( lastchat ) {
        if (self.loading == false) {
            $.ajax({
                url: "https://m.weddyday.com/api/chat.api.php",
                type: "GET",
                data: {
                    action: "getChat",
                    channel: self.channel,
                    last: lastchat,
                    token: self.getToken()
                },
                dataType: "json",
                beforeSend: function() {
                    if (lastchat == 0) {
                        $( "div#messages" ).empty();
                    }
                },
                xhr: function() {
                    if (lastchat == 0) {
                        return self.loadingProgress();
                    } else {
                        return $.ajaxSettings.xhr();
                    }
                }
            }).success( function( response ) {
                $.each( response.messages, function( id, message ) {
                    $( "div#messages" ).append( message );
                });
                if (response.last > 0) {
                    self.scrollChat();
                    self.lastchat = response.last;
                }
            }).fail( function( error ) {
                switch( error.status ) {
                    case 401: {
                        self.unsetUser();
                        self.loadSection( "login" );
                        break;
                    }
                    default: self.fatalError( error.status );
                }
            });
        }
    }

    this.scrollChat = function() {
        var client = window.innerHeight;
        var height = $( document ).height();
        if (self.scroll == true) {
            $.mobile.silentScroll( height - client );
        }
    }

    this.setChannel = function() {
        self.channel = this.value;
        self.toggleChatFilter();
        self.initializeChat();
    }

    this.setScroll = function() {
        self.scroll = true;
        $( "button[name='scroll']" ).fadeOut( 250 );
        self.scrollChat();
    }

    this.unsetScroll = function() {
        self.scroll = false;
        $( "button[name='scroll']" ).fadeIn( 250 );
    }

    this.sendChat = function() {
        $.ajax({
            url: "https://m.weddyday.com/api/chat.api.php",
            type: "POST",
            data: {
                action: "setChat",
                message: $( "textarea[name='message']" ).val().trim(),
                image: self.cache,
                channel: self.channel,
                token: self.getToken()
            },
            dataType: "json",
            beforeSend: function() {
                $( "input[name='image']" ).attr( "disabled", "disabled" );
                $( "button[type='submit']" ).attr( "disabled", "disabled" );
            },
            xhr: function() {
                if (self.cache != null) {
                    return self.uploadProgress();
                } else {
                    return self.loadingProgress();
                }
            }
        }).success( function() {
            self.setScroll();
            self.loadChat();
        }).fail( function( error ) {
            switch( error.status ) {
                case 400: {
                    switch( error.responseJSON.error ) {
                        case "message": {
                            $( "textarea[name='message']" ).focus();
                            self.vibrate( 500 );
                            break;
                        }
                        case "image": {
                            $( "button[name='image']" ).css({ background: "#C00000" });
                            setTimeout( function() {
                                $( "button[name='image']" ).removeAttr( "style" );
                            }, 1000 );
                            self.vibrate( 500 );
                            break;
                        }
                        default: self.fatalError( error.status );
                    }
                    break;
                }
                case 401: {
                    self.unsetUser();
                    self.loadSection( "login" );
                    break;
                }
                default: self.fatalError( error.status );
            }
        }).complete( function() {
            self.cache = null;
            $( "input[name='image']" ).removeAttr( "disabled" );
            $( "textarea[name='message']" ).val( null );
            $( "button[type='submit']" ).removeAttr( "disabled" );
        });
    }

    this.selectImage = function() {
        $( "input[name='image']" ).trigger( "click" );
    }

    this.loadImage = function() {
        var image = $( this ).prop( "files" )[0];
        var cache = new FileReader();
        cache.onload = function( event ) {
            self.cache = event.target.result;
            $( "button[name='image']" ).css({ background: "#236B9E" });
        };
        cache.readAsDataURL( image );
    }

    this.loadOutfit = function() {
        $.each( self.getStock(), function( section, stock ) {
            $( "span#stock-" + section ).text( self.formatNumber( stock ) );
        });
    }

    this.initializeProducts = function() {
        self.results = true;
        self.loading = false;
        self.loadProducts( 1 );
    }

    this.loadProducts = function( page ) {
        if ((self.loading == false) && (self.results == true)) {
            $.ajax({
                url: "https://m.weddyday.com/api/products.api.php",
                type: "GET",
                data: {
                    section: self.section,
                    filter: self.getProductFilter(),
                    page: page
                },
                dataType: "json",
                beforeSend: function() {
                    if (page == 1) {
                        $( "ul#products-" + self.section ).empty();
                    }
                },
                xhr: function() {
                    return self.loadingProgress();
                }
            }).success( function( response ) {
                $( "span#stock" ).text( self.formatNumber( response.total ) );
                $.each( response.products, function( id, product ) {
                    $( "ul#products-" + self.section ).append( product ).listview( "refresh" );
                });
                $.each( self.getMarkers(), function( id ) {
                    $( "a[data-marker='" + id + "']" ).addClass( "ui-btn-active" );
                });
                self.monitorImages();
                if (response.offset < response.total) {
                    self.results = true;
                    self.page = page + 1;
                } else {
                    self.results = false;
                }
            }).fail( function( error ) {
                self.fatalError( error.status );
            });
        }
    }

    this.getProductFilter = function() {
        var filter = {};
        $( "form#productfilter" ).serializeArray().map( function( input ) {
            filter[input.name] = input.value;
        });
        return JSON.stringify( filter );
    }

    this.updatePriceFilter = function() {
        var min = $( "input[name='price-min']" ).val();
        var max = $( "input[name='price-max']" ).val();
        $( "div#price span#min" ).html( self.formatNumber( min ) + " &euro;" );
        $( "div#price span#max" ).html( self.formatNumber( max ) + " &euro;" );
    }

    this.trackClick = function() {
        $.ajax({
            url: "https://m.weddyday.com/api/app.api.php",
            type: "POST",
            data: {
                action: "trackClick",
                product: $( this ).attr( "data-track" )
            },
            dataType: "json",
            xhr: function() {
                return self.loadingProgress();
            }
        });
    }

    this.initializeMap = function() {
        self.map = new GMaps({
            el: "#map",
            lat: 52.5243700,
            lng: 13.4105300,
            zoom: 10,
            mapTypeControl: false,
            streetViewControl: false,
            fullscreenControl: false,
            rotateControl: false,
            scaleControl: false,
            zoomControl: false,
            panControl: false
        });
        var styles = [
            { featureType: "all", stylers: [ { saturation: 0 }, { hue: "#236B9E" } ] },
            { featureType: "poi", stylers: [ { visibility: "off" } ] },
            { featureType: "water", stylers: [ { visibility: "simplified" }, { saturation: -60 } ] },
            { featureType: "road", stylers: [ { saturation: -70 } ] },
            { featureType: "transit", stylers: [ { visibility: "off" } ] }
        ];
        self.map.addStyle({
            styledMapName: "WeddyDay",
            mapTypeId: "weddyday",
            styles: styles
        });
        self.map.setStyle( "weddyday" );
        self.service = new google.maps.places.PlacesService( $( "div#detail" ).get( 0 ) );
    }

    this.loadMap = function() {
        if (typeof google == "object") {
            self.initializeMap();
            if (self.getLocation() != null) {
                self.loadLocation( self.getLocation() );
            }
        } else {
            $.ajax({
                url: "https://maps.googleapis.com/maps/api/js",
                type: "GET",
                data: {
                    libraries: "places",
                    key: "AIzaSyBCoYW1AQYBoW2pZJ5wUExXgTw2r1yTbJ8"
                },
                dataType: "script",
                xhr: function() {
                    return self.loadingProgress();
                }
            }).success( function() {
                self.initializeMap();
                if (self.getLocation() != null) {
                    self.loadLocation( self.getLocation() );
                }
            }).fail( function( error ) {
                self.fatalError( error.status );
            });
        }
    }

    this.setLocation = function() {
        var location = $( "input[name='location']" ).val().trim();
        if (location.length > 0) {
            self.toggleMapFilter();
            self.unloadPlaceDetail();
            self.loadLocation( location );
        }
    }

    this.loadLocation = function( location ) {
        if (typeof google == "object") {
            GMaps.geocode({
                address: location,
                callback: function( locations, status ) {
                    if (status == google.maps.GeocoderStatus.OK) {
                        var target = locations[0].geometry.location;
                        self.map.removeMarkers();
                        self.map.setCenter( target.lat(), target.lng() );
                        self.map.addMarker({
                            lat: target.lat(),
                            lng: target.lng(),
                            icon: "img/map/position.png",
                            clickable: false,
                            zIndex: 1000
                        });
                        self.loadPlaces( target.lat(), target.lng() );
                        if ((self.appUser() == true) && (location != self.getLocation())) {
                            self.setUser( "setLocation", location );
                        }
                    } else {
                        self.toggleMapFilter();
                    }
                }
            });
        } else {
            self.fatalError( 0 );
        }
    }

    this.loadPlaces = function( lat, lng ) {
        $.ajax({
            url: "https://m.weddyday.com/api/app.api.php",
            type: "GET",
            data: {
                action: "getPlaces",
                lat: lat,
                lng: lng,
                type: self.section
            },
            dataType: "json",
            beforeSend: function() {
                self.marker = null;
                $.each( self.markers, function() {
                    clearInterval( this );
                });
                self.markers = {};
            },
            xhr: function() {
                return self.loadingProgress();
            }
        }).success( function( response ) {
            $.each( response, function( i, place ) {
                if ($( "div#map-" + self.section ).length > 0) {
                    place.stars = 5;
                    self.addPremiumMarker( place, i );
                }
            });
            self.map.addLayer( "places", {
                location: new google.maps.LatLng( lat, lng ),
                keyword: self.getKeywords(),
                radius: 50000,
                nearbySearch: function( places, status ) {
                    if (status == google.maps.places.PlacesServiceStatus.OK) {
                        $.each( places, function( i, place ) {
                            if (self.mapMarker( place ) == false) {
                                self.markers[i] = setTimeout( function() {
                                    if ($( "div#map-" + self.section ).length > 0) {
                                        self.addMarker( place, i );
                                    }
                                }, 100 * i );
                            }
                        });
                    } else {
                        self.fatalError( 0 );
                    }
                }
            });
        }).fail( function( error ) {
            self.fatalError( error.status );
        });
    }

    this.addPremiumMarker = function( place, i ) {
        self.map.addMarker({
            lat: place.lat,
            lng: place.lng,
            icon: "img/map/premium/" + self.section + ".png",
            animation: google.maps.Animation.DROP,
            zIndex: 201 + i,
            click: function() {
                if (this != self.marker) {
                    if (self.marker != null) {
                        self.marker.setAnimation( null );
                    }
                    this.setAnimation( google.maps.Animation.BOUNCE );
                    self.map.setCenter( place.lat, place.lng );
                    self.loadPlaceDetail( place );
                    self.marker = this;
                } else {
                    self.unloadPlaceDetail();
                }
            }
        });
    }

    this.addMarker = function( place, i ) {
        self.map.addMarker({
            lat: place.geometry.location.lat(),
            lng: place.geometry.location.lng(),
            icon: "img/map/standard/" + self.section + ".png",
            animation: google.maps.Animation.DROP,
            zIndex: i,
            click: function() {
                if (this != self.marker) {
                    if (self.marker != null) {
                        self.marker.setAnimation( null );
                    }
                    this.setAnimation( google.maps.Animation.BOUNCE );
                    self.map.setCenter( place.geometry.location.lat(), place.geometry.location.lng() );
                    self.loadPlace( place.place_id );
                    self.marker = this;
                } else {
                    self.unloadPlaceDetail();
                }
            }
        });
    }

    this.mapMarker = function( place ) {
        var position = false;
        $.each( self.map.markers, function( i, marker ) {
            if (place.geometry.location.equals( marker.position )) {
                position = true;
            }
        });
        return position;
    }

    this.loadPlace = function( place ) {
        self.service.getDetails({ placeId: place }, function( details, status ) {
            if (status == google.maps.places.PlacesServiceStatus.OK) {
                var place = {};
                place.name = details.name;
                place.address = details.formatted_address;
                if (typeof details.international_phone_number == "string") {
                    place.phone = details.international_phone_number;
                }
                if (self.getPromotion( details.name ) != null) {
                    place.website = self.getPromotion( details.name );
                } else if (typeof details.website == "string") {
                    place.website = details.website;
                }
                if (typeof details.photos == "object") {
                    place.image = details.photos[0].getUrl({ "maxWidth": 500, "maxHeight": 500 });
                }
                place.lat = details.geometry.location.lat();
                place.lng = details.geometry.location.lng();
                if (typeof details.rating == "number") {
                    place.stars = details.rating;
                }
                if (typeof details.opening_hours == "object") {
                    place.open = details.opening_hours.open_now;
                }
                self.loadPlaceDetail( place );
            } else {
                self.fatalError( 0 );
            }
        });
    }

    this.loadPlaceDetail = function( place ) {
        var html = "<div data-role='navbar'><ul>";
        if (typeof place.phone == "string") {
            if (typeof place.open == "boolean") {
                if (place.open == true) {
                    html += "<li><a data-action='tel:" + place.phone + "' class='open ui-btn ui-btn-icon-left ui-icon-phone'>Anrufen</a></li>";
                } else {
                    html += "<li><a data-action='tel:" + place.phone + "' class='close ui-btn ui-btn-icon-left ui-icon-phone'>Anrufen</a></li>";
                }
            } else {
                html += "<li><a data-action='tel:" + place.phone + "' class='ui-btn ui-btn-icon-left ui-icon-phone'>Anrufen</a></li>";
            }
        }
        html += "<li><a data-link='https://www.google.de/maps?daddr=" + place.address + "' class='ui-btn ui-btn-icon-left ui-icon-navigation'>Navigieren</a></li>";
        if (typeof place.website == "string") {
            html += "<li><a data-link='" + place.website + "' class='ui-btn'>Zum Anbieter</a></li>";
        }
        html += "</ul></div><div class='overview'>";
        if (typeof place.image == "string") {
            html += "<img src='" + place.image + "' class='image maximize'>";
        }
        html += "<div class='details'>";
        if (typeof place.stars == "number") {
            html += "<div class='stars'>";
            for( var i = 1; i <= Math.round( place.stars ); i ++ ) {
                html += "<img src='img/map/star.png'>";
            }
            html += "</div>";
        }
        html += "<h1>" + place.name + "</h1>";
        html += "<p>" + place.address + "</p>";
        html += "</div></div>";
        $( "a#booking" ).animate({ bottom: "10.8em" }, 250 );
        $( "div#detail" ).html( html ).enhanceWithin().slideDown( 250 );
        self.monitorImages();
    }

    this.unloadPlaceDetail = function() {
        if (self.marker != null) {
            self.marker.setAnimation( null );
            self.marker = null;
            $( "a#booking" ).animate({ bottom: "2.4em" }, 250 );
            $( "div#detail" ).slideUp( 250 );
        }
    }

    this.getUser = function() {
        $.ajax({
            url: "https://m.weddyday.com/api/user.api.php",
            type: "GET",
            data: {
                action: "getUser",
                token: self.getToken()
            },
            dataType: "json",
            xhr: function() {
                return self.loadingProgress();
            }
        }).success( function( response ) {
            self.user = response;
            if (self.redirect != null) {
                self.loadSection( self.redirect );
                self.redirect = null;
            } else {
                self.loadSection( "main" );
            }
        }).fail( function() {
            self.unsetToken();
            self.loadSection( "main" );
        });
    };

    this.appUser = function() {
        if (Object.keys( self.user ).length > 0) {
            return true;
        } else {
            return false;
        }
    }

    this.getDate = function() {
        if (typeof self.user.date == "string") {
            if (self.user.date != "0000-00-00") {
                return self.user.date;
            } else {
                return null;
            }
        } else {
            return null;
        }
    }

    this.getLocation = function() {
        if (typeof self.user.location == "string") {
            if (self.user.location.length > 0) {
                return self.user.location;
            } else {
                return null;
            }
        } else {
            return null;
        }
    }

    this.getMarkers = function() {
        if (typeof self.user.markers == "object") {
            return self.user.markers;
        } else {
            return {};
        }
    }

    this.countMarkers = function() {
        return Object.keys( self.getMarkers() ).length;
    }

    this.getTodos = function() {
        if (typeof self.user.todos == "object") {
            return self.user.todos;
        } else {
            return {};
        }
    }

    this.countTodos = function() {
        return Object.keys( self.getTodos() ).length;
    }

    this.getBudgets = function() {
        if (typeof self.user.budgets == "object") {
            return self.user.budgets;
        } else {
            return {};
        }
    }

    this.countBudgets = function() {
        var budgets = 0;
        $.each( self.getBudgets(), function( id, budget ) {
            budgets += budget;
        });
        return budgets;
    }

    this.getGuests = function() {
        if (typeof self.user.guests == "object") {
            return self.user.guests;
        } else {
            return {};
        }
    }

    this.countGuests = function() {
        var guests = 0;
        $.each( self.getGuests(), function( id, guest ) {
            if (guest.match( /^\[(.*?)\]$/ ) == null) {
                guests += 1;
            }
        });
        return guests;
    }

    this.setUser = function( action, data ) {
        $.ajax({
            url: "https://m.weddyday.com/api/user.api.php",
            type: "POST",
            data: {
                action: action,
                data: data,
                token: self.getToken()
            },
            dataType: "json",
            xhr: function() {
                return self.loadingProgress();
            }
        }).success( function( response ) {
            self.user = response;
            if (action == "setMark") {
                $( "a[data-marker='" + data + "']" ).toggleClass( "ui-btn-active" );
                self.vibrate( 250 );
            } else if ((action == "setGuest") || (action == "unsetGuest")) {
                self.loadSection( "guests" );
            }
        }).fail( function( error ) {
            switch( error.status ) {
                case 400: {
                    if (action == "setLocation") {
                        self.toggleMapFilter();
                    } else if (action == "setGuest") {
                        $( "input[name='name']" ).focus();
                    }
                    break;
                }
                case 401: {
                    self.unsetUser();
                    self.loadSection( "login" );
                    break;
                }
                default: self.fatalError( error.status );
            }
        });
    };

    this.setDate = function() {
        var date = $( "input[name^='date']" ).map( function( datepart ) {
            if (datepart == 2) {
                var part = ( "0000" + $( this ).val() ).slice( -4 );
            } else {
                var part = ( "00" + $( this ).val() ).slice( -2 );
            }
            $( this ).val( part );
            return part;
        }).get();
        self.setUser( "setDate", date[2] + "-" + date[1] + "-" + date[0] );
    }

    this.setMarker = function() {
        var marker = $( this ).attr( "data-marker" );
        if (self.appUser() == true) {
            self.setUser( "setMark", marker );
        }
    }

    this.setTodo = function() {
        var todo = $( this ).attr( "data-todo" );
        self.setUser( "setTodo", todo );
    }

    this.setBudget = function() {
        var budget = {};
        budget.budget = $( this ).attr( "name" );
        budget.amount = $( this ).val().replace( ".", "" ).replace( ",", "" );
        if ((isNaN( parseInt( budget.amount ) ) == true) || (parseInt( budget.amount ) == 0)) {
            budget.amount = 0;
        }
        self.setUser( "setBudget", JSON.stringify( budget ) );
        $( this ).val( self.formatNumber( budget.amount ) );
        self.calculateBudget();
        $( "a[data-budget]" ).removeClass( "ui-btn-active" );
    }

    this.setGuest = function() {
        var type = $( "input[name='type']:checked" ).val();
        var name = $( "input[name='name']" ).val().trim();
        if (type == "guest") {
            self.setUser( "setGuest", name );
        } else if (type == "group") {
            self.setUser( "setGuest", "[" + name + "]" );
        }
    }

    this.unsetGuest = function() {
        var guest = $( this ).attr( "data-guest" );
        self.setUser( "unsetGuest", guest );
    }

    this.sortGuests = function() {
        var guests = {};
        $( "ul#guests" ).find( "li" ).each( function( guest ) {
            $( this ).find( "a[data-guest]" ).attr( "data-guest", guest );
            guests[guest] = $( this ).find( "a[data-name]" ).attr( "data-name" );
        });
        self.setUser( "sortGuests", JSON.stringify( guests ) );
        $( "ul#guests" ).listview( "refresh" );
    }

    this.unsetUser = function() {
        self.user = {};
        self.unsetToken();
    }

    this.loginUser = function() {
        $.ajax({
            url: "https://m.weddyday.com/api/user.api.php",
            type: "POST",
            data: {
                action: "loginUser",
                username: $( "input[name='username']" ).val().trim(),
                password: $( "input[name='password']" ).val().trim()
            },
            dataType: "json",
            beforeSend: function() {
                self.resetErrors();
                $( "button[type='submit']" ).attr( "disabled", "disabled" );
            },
            xhr: function() {
                return self.loadingProgress();
            }
        }).success( function( response ) {
            self.setToken( response.token );
            self.getUser();
        }).fail( function( error ) {
            switch( error.status ) {
                case 400: {
                    switch( error.responseJSON.error ) {
                        case "username": {
                            var error = "Benutzername fehlerhaft";
                            $( "input[name='username']" ).addClass( "error" ).focus();
                            $( "input[name='username']" ).parent().after( "<label class='error'>" + error + "</label>" );
                            self.vibrate( 500 );
                            break;
                        }
                        case "password": {
                            var error = "Passwort fehlerhaft";
                            $( "input[name='password']" ).addClass( "error" ).focus();
                            $( "input[name='password']" ).parent().after( "<label class='error'>" + error + "</label>" );
                            self.vibrate( 500 );
                            break;
                        }
                        default: self.fatalError( error.status );
                    }
                    break;
                }
                case 401: {
                    var error = "Login fehlgeschlagen";
                    $( "input[name='password']" ).parent().after( "<label class='error'>" + error + "</label>" );
                    self.vibrate( 500 );
                    break;
                }
                default: self.fatalError( error.status );
            }
        }).complete( function() {
            $( "button[type='submit']" ).removeAttr( "disabled" );
        });
    }

    this.logoutUser = function() {
        self.unsetUser();
    }

    this.restoreUser = function() {
        $.ajax({
            url: "https://m.weddyday.com/api/user.api.php",
            type: "POST",
            data: {
                action: "restoreUser",
                username: $( "input[name='username']" ).val().trim(),
                email: $( "input[name='email']" ).val().trim(),
                code: $( "input[name='code']" ).val().trim(),
                password: $( "input[name='password']" ).val().trim(),
                passcheck: $( "input[name='passcheck']" ).val().trim()
            },
            dataType: "json",
            beforeSend: function() {
                self.resetErrors();
                $( "button[type='submit']" ).attr( "disabled", "disabled" );
            },
            xhr: function() {
                return self.loadingProgress();
            }
        }).success( function( response ) {
            if (typeof response.validity == "string") {
                $( "div#step-1" ).fadeOut( 250, function() {
                    $( "div#step-2" ).fadeIn( 250, function() {
                        $( "input[name='code']" ).countdown( response.validity ).on( "update.countdown", function( event ) {
                            $( "input[name='code']" ).attr( "placeholder", event.strftime( "PIN-Code - läuft ab in %M:%S" ) );
                        }).on( "finish.countdown", function() {
                            self.loadSection( "restore" );
                        });
                    });
                });
            } else if (typeof response.token == "string") {
                self.setToken( response.token );
                self.getUser();
            }
        }).fail( function( error ) {
            switch( error.status ) {
                case 400: {
                    switch( error.responseJSON.error ) {
                        case "username": {
                            var error = "Benutzername fehlerhaft";
                            $( "input[name='username']" ).addClass( "error" ).focus();
                            $( "input[name='username']" ).parent().after( "<label class='error'>" + error + "</label>" );
                            self.vibrate( 500 );
                            break;
                        }
                        case "email": {
                            var error = "E-Mail fehlerhaft";
                            $( "input[name='email']" ).addClass( "error" ).focus();
                            $( "input[name='email']" ).parent().after( "<label class='error'>" + error + "</label>" );
                            self.vibrate( 500 );
                            break;
                        }
                        case "code": {
                            var error = "PIN-Code fehlerhaft";
                            $( "input[name='code']" ).addClass( "error" ).focus();
                            $( "input[name='code']" ).parent().after( "<label class='error'>" + error + "</label>" );
                            self.vibrate( 500 );
                            break;
                        }
                        case "password": {
                            var error = "Mindestens 5 Buchenstaben, Zahlen und Sonderzeichen";
                            $( "input[name='password']" ).addClass( "error" ).focus();
                            $( "input[name='password']" ).parent().after( "<label class='error'>" + error + "</label>" );
                            self.vibrate( 500 );
                            break;
                        }
                        case "passcheck": {
                            var error = "Passwort stimmt nicht überein";
                            $( "input[name='passcheck']" ).addClass( "error" ).focus();
                            $( "input[name='passcheck']" ).parent().after( "<label class='error'>" + error + "</label>" );
                            self.vibrate( 500 );
                            break;
                        }
                        default: self.fatalError( error.status );
                    }
                    break;
                }
                case 401: {
                    var error = "Benutzer existiert nicht";
                    $( "input[name='email']" ).parent().after( "<label class='error'>" + error + "</label>" );
                    self.vibrate( 500 );
                    break;
                }
                default: self.fatalError( error.status );
            }
        }).complete( function() {
            $( "button[type='submit']" ).removeAttr( "disabled" );
        });
    }

    this.registerUser = function() {
        $.ajax({
            url: "https://m.weddyday.com/api/user.api.php",
            type: "POST",
            data: {
                action: "registerUser",
                username: $( "input[name='username']" ).val().trim(),
                email: $( "input[name='email']" ).val().trim(),
                password: $( "input[name='password']" ).val().trim(),
                passcheck: $( "input[name='passcheck']" ).val().trim()
            },
            dataType: "json",
            beforeSend: function() {
                self.resetErrors();
                $( "button[type='submit']" ).attr( "disabled", "disabled" );
            },
            xhr: function() {
                return self.loadingProgress();
            }
        }).success( function( response ) {
            self.setToken( response.token );
            self.getUser();
        }).fail( function( error ) {
            switch( error.status ) {
                case 400: {
                    switch( error.responseJSON.error ) {
                        case "username": {
                            var error = "Mindestens 5 Buchenstaben und Zahlen";
                            $( "input[name='username']" ).addClass( "error" ).focus();
                            $( "input[name='username']" ).parent().after( "<label class='error'>" + error + "</label>" );
                            self.vibrate( 500 );
                            break;
                        }
                        case "userexist": {
                            var error = "Benutzername vergeben";
                            $( "input[name='username']" ).addClass( "error" ).focus();
                            $( "input[name='username']" ).parent().after( "<label class='error'>" + error + "</label>" );
                            self.vibrate( 500 );
                            break;
                        }
                        case "email": {
                            var error = "E-Mail fehlerhaft";
                            $( "input[name='email']" ).addClass( "error" ).focus();
                            $( "input[name='email']" ).parent().after( "<label class='error'>" + error + "</label>" );
                            self.vibrate( 500 );
                            break;
                        }
                        case "password": {
                            var error = "Mindestens 5 Buchenstaben, Zahlen und Sonderzeichen";
                            $( "input[name='password']" ).addClass( "error" ).focus();
                            $( "input[name='password']" ).parent().after( "<label class='error'>" + error + "</label>" );
                            self.vibrate( 500 );
                            break;
                        }
                        case "passcheck": {
                            var error = "Passwort stimmt nicht überein";
                            $( "input[name='passcheck']" ).addClass( "error" ).focus();
                            $( "input[name='passcheck']" ).parent().after( "<label class='error'>" + error + "</label>" );
                            self.vibrate( 500 );
                            break;
                        }
                        default: self.fatalError( error.status );
                    }
                    break;
                }
                case 500: {
                    var error = "Anmeldung fehlgeschlagen";
                    $( "input[name='passcheck']" ).parent().after( "<label class='error'>" + error + "</label>" );
                    self.vibrate( 500 );
                    break;
                }
                default: self.fatalError( error.status );
            }
        }).complete( function() {
            $( "button[type='submit']" ).removeAttr( "disabled" );
        });
    }

    this.resetErrors = function() {
        $( "input.error" ).removeClass( "error" );
        $( "label.error" ).remove();
    }

    this.getToken = function() {
        return localStorage.getItem( "token" );
    }

    this.setToken = function( token ) {
        localStorage.setItem( "token", token );
    }

    this.unsetToken = function() {
        localStorage.removeItem( "token" );
    }

    this.vibrate = function( time ) {
        if ("vibrate" in navigator) {
            navigator.vibrate( time );
        }
    }

    this.loadingProgress = function() {
        var request = $.ajaxSettings.xhr();
        request.onloadstart = function() {
            $.mobile.loading( "show" );
            self.loading = true;
        };
        request.onloadend = function() {
            if ($( "h1.error" ).length == 0) {
                $.mobile.loading( "hide" );
            }
            self.loading = false;
        };
        return request;
    }

    this.uploadProgress = function() {
        var request = $.ajaxSettings.xhr();
        request.upload.onprogress = function( event ) {
            var progress = Math.floor( ( event.loaded / event.total ) * 100 );
            if (progress < 100) {
                $( "button[name='image']" ).css({
                    background: "linear-gradient(to top, #22AADD " + progress + "%, #236B9E 0%)"
                });
            } else {
                $( "button[name='image']" ).removeAttr( "style" );
            }
        };
        return request;
    }

    this.fatalError = function( error ) {
        $.mobile.loading( "show", {
            html: "<h1 class='error'>" + self.getError( error ) + "</h1>",
            text: self.getError( error ),
            textVisible: true,
            textonly: true
        });
        self.vibrate( 500 );
        clearInterval( self.error );
        self.error = setTimeout( function() {
            $.mobile.loading( "hide" );
        }, 3000 );
    }

    this.getError = function( error ) {
        switch( error ) {
            case 0: {
                return "Keine Serververbindung";
                break;
            }
            default: {
                return "Interner Serverfehler";
            }
        }
    }

    this.formatNumber = function( number ) {
        return number.toString().replace( /\B(?=(\d{3})+(?!\d))/, "." );
    }

    this.monitorImages = function() {
        $( "img.maximize" ).on( "error", function() {
            $( this ).attr( "src", "img/icons/image.png" ).removeClass( "maximize" );
        });
    }

    this.maximizeImage = function() {
        if ($( "div#maximize" ).length > 0) {
            $( "div#maximize span" ).fadeOut( 250, function() {
                $( "div#maximize" ).animate({ top: "50%", left: "50%", width: "0%", height: "0%" }, 250, function() {
                    $( this ).remove();
                });
            });
        } else {
            var image = $( this ).attr( "src" );
            $( "div[data-role='content']" ).append( "<div id='maximize'><img src='" + image + "'></img><span></span></div>" );
            $( "div#maximize" ).animate({ top: "0%", left: "0%", width: "100%", height: "100%" }, 250, function() {
                $( "div#maximize span" ).fadeIn( 250 );
            });
        }
    }

    this.toggleFilter = function( filter ) {
        if ($( "form#" + filter ).is( ":hidden" ) == true) {
            $( "a[data-filter='" + filter + "']" ).addClass( "ui-btn-active" );
            $( "form#" + filter ).slideDown( 250, function() {
                $( "a[data-filter='" + filter + "']" ).removeClass( "ui-icon-carat-d" );
                $( "a[data-filter='" + filter + "']" ).addClass( "ui-icon-carat-u" );
                if (filter == "mapfilter") {
                    $( "input[name='location']" ).focus();
                }
            });
        } else {
            $( "a[data-filter='" + filter + "']" ).removeClass( "ui-btn-active" );
            $( "form#" + filter ).slideUp( 250, function() {
                $( "a[data-filter='" + filter + "']" ).removeClass( "ui-icon-carat-u" );
                $( "a[data-filter='" + filter + "']" ).addClass( "ui-icon-carat-d" );
                if (filter == "mapfilter") {
                    $( "input[name='location']" ).val( null ).blur();
                }
            });
        }
    }

    this.toggleProductFilter = function() {
        self.toggleFilter( "productfilter" );
    }

    this.toggleMapFilter = function() {
        self.toggleFilter( "mapfilter" );
    }

    this.toggleChatFilter = function() {
        self.toggleFilter( "chatfilter" );
        $( "input#channel-" + self.channel ).prop( "checked", true ).checkboxradio( "refresh" );
    }

    this.initializeApp();

}